/*
 * sbuf.h -- simple text editor buffer routines
 *
 * Part of Publib.  See manpage for more information.
 * "@(#)publib-sbuf:sbuf.h,v 1.1.1.1 1994/08/28 17:12:05 liw Exp"
 */


#ifndef __publib_sbuf_included
#define __publib_sbuf_included


/*
 * Includes for types defined elsewhere.
 * These are all standard headers.
 */

#include <stddef.h>		/* need size_t */


struct __sbufmark {
	unsigned inuse:1;	/* is this mark in use? */
	unsigned dirty:1;	/* has text in mark been modified? */
	long begin;		/* where the buffer begins ... */
	long len;		/* ...and how long it is */
};


struct __sbuf {
	char *block;
	unsigned locked:1;
	size_t gappos;
	size_t alloc;
	size_t len;

	struct __sbufmark *marks;
	size_t markalloc;
};
typedef struct __sbuf Sbuf;

/*
 * This is how the client sees a struct __bufmark.  We can't use a
 * pointer directly, since the struct __bufmarks may move because of
 * realloc, and we can't assume that the client will be able to update
 * his pointers.
 */
typedef struct {
	Sbuf *buf;
	size_t mark;
} Sbufmark;


/*
 * The validation functions shouldn't generate any code when NDEBUG is
 * defined.  This is the same behaviour as assert has.  Note that the
 * prototypes and definitions of these functions are protected against
 * the macros by virtue of having the function names within parentheses.
 */
#ifdef NDEBUG
#define sbuf_validate(buf)	((void)0)
#define sbuf_validate_mark(m)	((void)0)
#endif


/*
 * Let's use the same NDEBUG flag to turn on a macro version of sbuf_charat.
 * _NO_ error checking for this baby.  Use the non-macro version for that.
 */
#ifdef NDEBUG
#include <stdio.h> /* need EOF */
#define sbuf_charat(buf, pos) \
	((pos) == (buf)->len \
		?  EOF \
		: (pos) < (buf)->gappos \
			? (buf)->block[(pos)] \
			: (buf)->block[(pos) + (buf)->gappos])
#endif


void sbufgap_movegap(Sbuf *, size_t);

void sbuf_insert(Sbuf *, size_t, const char *, size_t);
void sbuf_delete(Sbuf *, size_t, size_t);
char *sbuf_lock(Sbuf *);
void sbuf_unlock(Sbuf *);

Sbuf *sbuf_create(void);
void sbuf_destroy(Sbuf *);
void (sbuf_validate)(Sbuf *);

Sbufmark *sbuf_mark(Sbuf *, long, long);
void sbuf_remark(Sbufmark *, long, long);
void sbuf_unmark(Sbufmark *);
long sbuf_mark_begin(Sbufmark *);
long sbuf_mark_end(Sbufmark *);
long sbuf_mark_len(Sbufmark *);
int sbuf_mark_is_dirty(Sbufmark *);
void sbuf_mark_set_dirty(Sbufmark *, int);
void (sbuf_validate_mark)(Sbufmark *);

int sbuf_strchange(Sbufmark *, const char *, size_t);
int sbuf_change(Sbufmark *, Sbufmark *);
int (sbuf_charat)(Sbuf *, long);
void sbuf_strat(char *, Sbufmark *);

int sbuf_insert_file(Sbufmark *, FILE *);
int sbuf_write_to_file(Sbufmark *, FILE *);

long sbuf_length(Sbuf *);

#endif
